@@ -39,56 +39,68 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "r300_program.h"
 #include "program_instruction.h"
 
+#if SWIZZLE_X != VSF_IN_COMPONENT_X || \
+    SWIZZLE_Y != VSF_IN_COMPONENT_Y || \
+    SWIZZLE_Z != VSF_IN_COMPONENT_Z || \
+    SWIZZLE_W != VSF_IN_COMPONENT_W || \
+    SWIZZLE_ZERO != VSF_IN_COMPONENT_ZERO || \
+    SWIZZLE_ONE != VSF_IN_COMPONENT_ONE || \
+    WRITEMASK_X != VSF_FLAG_X || \
+    WRITEMASK_Y != VSF_FLAG_Y || \
+    WRITEMASK_Z != VSF_FLAG_Z || \
+    WRITEMASK_W != VSF_FLAG_W
+#error Cannot change these!
+#endif
+    
 #define SCALAR_FLAG (1<<31)
 #define FLAG_MASK (1<<31)
 #define OP_MASK	(0xf)  /* we are unlikely to have more than 15 */
-#define OPN(operator, ip, op) {#operator, OPCODE_##operator, ip, op}
+#define OPN(operator, ip) {#operator, OPCODE_##operator, ip}
 
 static struct{
 	char *name;
 	int opcode;
 	unsigned long ip; /* number of input operands and flags */
-	unsigned long op;
 }op_names[]={
-	OPN(ABS, 1, 1),
-	OPN(ADD, 2, 1),
-	OPN(ARL, 1, 1|SCALAR_FLAG),
-	OPN(DP3, 2, 3|SCALAR_FLAG),
-	OPN(DP4, 2, 3|SCALAR_FLAG),
-	OPN(DPH, 2, 3|SCALAR_FLAG),
-	OPN(DST, 2, 1),
-	OPN(EX2, 1|SCALAR_FLAG, 4|SCALAR_FLAG),
-	OPN(EXP, 1|SCALAR_FLAG, 1),
-	OPN(FLR, 1, 1),
-	OPN(FRC, 1, 1),
-	OPN(LG2, 1|SCALAR_FLAG, 4|SCALAR_FLAG),
-	OPN(LIT, 1, 1),
-	OPN(LOG, 1|SCALAR_FLAG, 1),
-	OPN(MAD, 3, 1),
-	OPN(MAX, 2, 1),
-	OPN(MIN, 2, 1),
-	OPN(MOV, 1, 1),
-	OPN(MUL, 2, 1),
-	OPN(POW, 2|SCALAR_FLAG, 4|SCALAR_FLAG),
-	OPN(RCP, 1|SCALAR_FLAG, 4|SCALAR_FLAG),
-	OPN(RSQ, 1|SCALAR_FLAG, 4|SCALAR_FLAG),
-	OPN(SGE, 2, 1),
-	OPN(SLT, 2, 1),
-	OPN(SUB, 2, 1),
-	OPN(SWZ, 1, 1),
-	OPN(XPD, 2, 1),
-	OPN(RCC, 0, 0), //extra
-	OPN(PRINT, 0, 0),
-	OPN(END, 0, 0),
+	OPN(ABS, 1),
+	OPN(ADD, 2),
+	OPN(ARL, 1|SCALAR_FLAG),
+	OPN(DP3, 2),
+	OPN(DP4, 2),
+	OPN(DPH, 2),
+	OPN(DST, 2),
+	OPN(EX2, 1|SCALAR_FLAG),
+	OPN(EXP, 1|SCALAR_FLAG),
+	OPN(FLR, 1),
+	OPN(FRC, 1),
+	OPN(LG2, 1|SCALAR_FLAG),
+	OPN(LIT, 1),
+	OPN(LOG, 1|SCALAR_FLAG),
+	OPN(MAD, 3),
+	OPN(MAX, 2),
+	OPN(MIN, 2),
+	OPN(MOV, 1),
+	OPN(MUL, 2),
+	OPN(POW, 2|SCALAR_FLAG),
+	OPN(RCP, 1|SCALAR_FLAG),
+	OPN(RSQ, 1|SCALAR_FLAG),
+	OPN(SGE, 2),
+	OPN(SLT, 2),
+	OPN(SUB, 2),
+	OPN(SWZ, 1),
+	OPN(XPD, 2),
+	OPN(RCC, 0), //extra
+	OPN(PRINT, 0),
+	OPN(END, 0),
 };
 #undef OPN
 	
 int r300VertexProgUpdateParams(GLcontext *ctx, struct r300_vertex_program *vp, float *dst)
 {
 	int pi;
-	struct vertex_program *mesa_vp=(void *)vp;
+	struct gl_vertex_program *mesa_vp = &vp->mesa_program;
 	float *dst_o=dst;
-        struct program_parameter_list *paramList;
+        struct gl_program_parameter_list *paramList;
 	
 	if (mesa_vp->IsNVProgram) {
 		_mesa_init_vp_per_primitive_registers(ctx);
@@ -134,14 +146,8 @@ int r300VertexProgUpdateParams(GLcontext *ctx, struct r300_vertex_program *vp, f
 		
 static unsigned long t_dst_mask(GLuint mask)
 {
-	unsigned long flags=0;
-	
-	if(mask & WRITEMASK_X) flags |= VSF_FLAG_X;
-	if(mask & WRITEMASK_Y) flags |= VSF_FLAG_Y;
-	if(mask & WRITEMASK_Z) flags |= VSF_FLAG_Z;
-	if(mask & WRITEMASK_W) flags |= VSF_FLAG_W;
-	
-	return flags;
+	/* WRITEMASK_* is equivalent to VSF_FLAG_* */
+	return mask & VSF_FLAG_ALL;
 }
 
 static unsigned long t_dst_class(enum register_file file)
@@ -211,19 +217,10 @@ static unsigned long t_src_class(enum register_file file)
 	}
 }
 
-static unsigned long t_swizzle(GLubyte swizzle)
+static __inline unsigned long t_swizzle(GLubyte swizzle)
 {
-	switch(swizzle){
-		case SWIZZLE_X: return VSF_IN_COMPONENT_X;
-		case SWIZZLE_Y: return VSF_IN_COMPONENT_Y;
-		case SWIZZLE_Z: return VSF_IN_COMPONENT_Z;
-		case SWIZZLE_W: return VSF_IN_COMPONENT_W;
-		case SWIZZLE_ZERO: return VSF_IN_COMPONENT_ZERO;
-		case SWIZZLE_ONE: return VSF_IN_COMPONENT_ONE;
-		default:
-			fprintf(stderr, "problem in %s", __FUNCTION__);
-			exit(0);
-	}
+/* this is in fact a NOP as the Mesa SWIZZLE_* are all identical to VSF_IN_COMPONENT_* */
+	return swizzle;
 }
 
 #if 0
@@ -263,20 +260,26 @@ static unsigned long t_src_index(struct r300_vertex_program *vp, struct prog_src
 		
 		return vp->inputs[src->Index];
 	}else{
+		if (src->Index < 0) {
+			fprintf(stderr, "WARNING negative offsets for indirect addressing do not work\n");
+			return 0;
+		}
 		return src->Index;
 	}
 }
 
 static unsigned long t_src(struct r300_vertex_program *vp, struct prog_src_register *src)
 {
-	
+	/* src->NegateBase uses the NEGATE_ flags from program_instruction.h,
+	 * which equal our VSF_FLAGS_ values, so it's safe to just pass it here.
+	 */
 	return MAKE_VSF_SOURCE(t_src_index(vp, src),
 				t_swizzle(GET_SWZ(src->Swizzle, 0)),
 				t_swizzle(GET_SWZ(src->Swizzle, 1)),
 				t_swizzle(GET_SWZ(src->Swizzle, 2)),
 				t_swizzle(GET_SWZ(src->Swizzle, 3)),
 				t_src_class(src->File),
-				src->NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src->RelAddr << 4);
+				src->NegateBase) | (src->RelAddr << 4);
 }
 
 static unsigned long t_src_scalar(struct r300_vertex_program *vp, struct prog_src_register *src)
@@ -288,13 +291,14 @@ static unsigned long t_src_scalar(struct r300_vertex_program *vp, struct prog_sr
 				t_swizzle(GET_SWZ(src->Swizzle, 0)),
 				t_swizzle(GET_SWZ(src->Swizzle, 0)),
 				t_src_class(src->File),
-				src->NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+				src->NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src->RelAddr << 4);
 }
 
 static unsigned long t_opcode(enum prog_opcode opcode)
 {
 
 	switch(opcode){
+		case OPCODE_ARL: return R300_VPI_OUT_OP_ARL;
 		case OPCODE_DST: return R300_VPI_OUT_OP_DST;
 		case OPCODE_EX2: return R300_VPI_OUT_OP_EX2;
 		case OPCODE_EXP: return R300_VPI_OUT_OP_EXP;
@@ -332,41 +336,41 @@ static unsigned long op_operands(enum prog_opcode opcode)
 }
 
 /* TODO: Get rid of t_src_class call */
-#define CMP_SRCS(a, b) (a.Index != b.Index && \
+#define CMP_SRCS(a, b) ((a.RelAddr != b.RelAddr) || (a.Index != b.Index && \
 		       ((t_src_class(a.File) == VSF_IN_CLASS_PARAM && \
 			 t_src_class(b.File) == VSF_IN_CLASS_PARAM) || \
 			(t_src_class(a.File) == VSF_IN_CLASS_ATTR && \
-			 t_src_class(b.File) == VSF_IN_CLASS_ATTR))) \
+			 t_src_class(b.File) == VSF_IN_CLASS_ATTR)))) \
 			 
-#define ZERO_SRC_0 MAKE_VSF_SOURCE(t_src_index(vp, &src[0]), \
-				   SWIZZLE_ZERO, SWIZZLE_ZERO, \
-				   SWIZZLE_ZERO, SWIZZLE_ZERO, \
-				   t_src_class(src[0].File), VSF_FLAG_NONE)
+#define ZERO_SRC_0 (MAKE_VSF_SOURCE(t_src_index(vp, &src[0]), \
+				    SWIZZLE_ZERO, SWIZZLE_ZERO, \
+				    SWIZZLE_ZERO, SWIZZLE_ZERO, \
+				    t_src_class(src[0].File), VSF_FLAG_NONE) | (src[0].RelAddr << 4))
 				   
-#define ZERO_SRC_1 MAKE_VSF_SOURCE(t_src_index(vp, &src[1]), \
-				   SWIZZLE_ZERO, SWIZZLE_ZERO, \
-				   SWIZZLE_ZERO, SWIZZLE_ZERO, \
-				   t_src_class(src[1].File), VSF_FLAG_NONE)
+#define ZERO_SRC_1 (MAKE_VSF_SOURCE(t_src_index(vp, &src[1]), \
+				    SWIZZLE_ZERO, SWIZZLE_ZERO, \
+				    SWIZZLE_ZERO, SWIZZLE_ZERO, \
+				    t_src_class(src[1].File), VSF_FLAG_NONE) | (src[1].RelAddr << 4))
 
-#define ZERO_SRC_2 MAKE_VSF_SOURCE(t_src_index(vp, &src[2]), \
-				   SWIZZLE_ZERO, SWIZZLE_ZERO, \
-				   SWIZZLE_ZERO, SWIZZLE_ZERO, \
-				   t_src_class(src[2].File), VSF_FLAG_NONE)
+#define ZERO_SRC_2 (MAKE_VSF_SOURCE(t_src_index(vp, &src[2]), \
+				    SWIZZLE_ZERO, SWIZZLE_ZERO, \
+				    SWIZZLE_ZERO, SWIZZLE_ZERO, \
+				    t_src_class(src[2].File), VSF_FLAG_NONE) | (src[2].RelAddr << 4))
 				   
-#define ONE_SRC_0 MAKE_VSF_SOURCE(t_src_index(vp, &src[0]), \
-				   SWIZZLE_ONE, SWIZZLE_ONE, \
-				   SWIZZLE_ONE, SWIZZLE_ONE, \
-				   t_src_class(src[0].File), VSF_FLAG_NONE)
+#define ONE_SRC_0 (MAKE_VSF_SOURCE(t_src_index(vp, &src[0]), \
+				    SWIZZLE_ONE, SWIZZLE_ONE, \
+				    SWIZZLE_ONE, SWIZZLE_ONE, \
+				    t_src_class(src[0].File), VSF_FLAG_NONE) | (src[0].RelAddr << 4))
 				   
-#define ONE_SRC_1 MAKE_VSF_SOURCE(t_src_index(vp, &src[1]), \
-				   SWIZZLE_ONE, SWIZZLE_ONE, \
-				   SWIZZLE_ONE, SWIZZLE_ONE, \
-				   t_src_class(src[1].File), VSF_FLAG_NONE)
+#define ONE_SRC_1 (MAKE_VSF_SOURCE(t_src_index(vp, &src[1]), \
+				    SWIZZLE_ONE, SWIZZLE_ONE, \
+				    SWIZZLE_ONE, SWIZZLE_ONE, \
+				    t_src_class(src[1].File), VSF_FLAG_NONE) | (src[1].RelAddr << 4))
 
-#define ONE_SRC_2 MAKE_VSF_SOURCE(t_src_index(vp, &src[2]), \
-				   SWIZZLE_ONE, SWIZZLE_ONE, \
-				   SWIZZLE_ONE, SWIZZLE_ONE, \
-				   t_src_class(src[2].File), VSF_FLAG_NONE)
+#define ONE_SRC_2 (MAKE_VSF_SOURCE(t_src_index(vp, &src[2]), \
+				    SWIZZLE_ONE, SWIZZLE_ONE, \
+				    SWIZZLE_ONE, SWIZZLE_ONE, \
+				    t_src_class(src[2].File), VSF_FLAG_NONE) | (src[2].RelAddr << 4))
 				   
 /* DP4 version seems to trigger some hw peculiarity */
 //#define PREFER_DP4
@@ -382,7 +386,7 @@ static unsigned long op_operands(enum prog_opcode opcode)
 
 void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 {
-	struct vertex_program *mesa_vp=(void *)vp;
+	struct gl_vertex_program *mesa_vp= &vp->mesa_program;
 	struct prog_instruction *vpi;
 	int i, cur_reg=0;
 	VERTEX_SHADER_INSTRUCTION *o_inst;
@@ -395,6 +399,9 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 	int u_temp_i=VSF_MAX_FRAGMENT_TEMPS-1;
 	struct prog_src_register src[3];
 
+	if (mesa_vp->Base.NumInstructions == 0)
+		return;
+
 	if (getenv("R300_VP_SAFETY")) {
 		WARN_ONCE("R300_VP_SAFETY enabled.\n");
 		
@@ -429,7 +436,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 	}
 	
 	if (mesa_vp->IsPositionInvariant) {
-		struct program_parameter_list *paramList;
+		struct gl_program_parameter_list *paramList;
 		GLint tokens[6] = { STATE_MATRIX, STATE_MVP, 0, 0, 0, STATE_MATRIX };
 
 #ifdef PREFER_DP4
@@ -576,7 +583,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 				o_inst->src1=MAKE_VSF_SOURCE(t_src_index(vp, &src[2]),
 						SWIZZLE_X, SWIZZLE_Y,
 						SWIZZLE_Z, SWIZZLE_W,
-						t_src_class(src[2].File), VSF_FLAG_NONE);
+						t_src_class(src[2].File), VSF_FLAG_NONE) | (src[2].RelAddr << 4);
 
 				o_inst->src2=ZERO_SRC_2;
 				o_inst->src3=ZERO_SRC_2;
@@ -584,6 +591,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 						
 				src[2].File=PROGRAM_TEMPORARY;
 				src[2].Index=u_temp_i;
+				src[2].RelAddr=0;
 				u_temp_i--;
 			}
 			
@@ -597,7 +605,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 				o_inst->src1=MAKE_VSF_SOURCE(t_src_index(vp, &src[0]),
 						SWIZZLE_X, SWIZZLE_Y,
 						SWIZZLE_Z, SWIZZLE_W,
-						t_src_class(src[0].File), VSF_FLAG_NONE);
+						t_src_class(src[0].File), VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 
 				o_inst->src2=ZERO_SRC_0;
 				o_inst->src3=ZERO_SRC_0;
@@ -605,20 +613,13 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 						
 				src[0].File=PROGRAM_TEMPORARY;
 				src[0].Index=u_temp_i;
+				src[0].RelAddr=0;
 				u_temp_i--;
 			}
 		}
 		
 		/* These ops need special handling. */
 		switch(vpi->Opcode){
-		case OPCODE_ARL:
-			o_inst->op=MAKE_VSF_OP(R300_VPI_OUT_OP_ARL, t_dst_index(vp, &vpi->DstReg),
-					t_dst_mask(vpi->DstReg.WriteMask), t_dst_class(vpi->DstReg.File));
-			o_inst->src1=t_src_scalar(vp, &src[0]);
-			o_inst->src2=ZERO_SRC_0;
-			o_inst->src3=ZERO_SRC_0;
-			goto next;
-			
 		case OPCODE_POW:
 			o_inst->op=MAKE_VSF_OP(R300_VPI_OUT_OP_POW, t_dst_index(vp, &vpi->DstReg),
 					t_dst_mask(vpi->DstReg.WriteMask), t_dst_class(vpi->DstReg.File));
@@ -628,6 +629,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 			goto next;
 			
 		case OPCODE_MOV://ADD RESULT 1.X Y Z W PARAM 0{} {X Y Z W} PARAM 0{} {ZERO ZERO ZERO ZERO} 
+		case OPCODE_SWZ:
 #if 1
 			o_inst->op=MAKE_VSF_OP(R300_VPI_OUT_OP_ADD, t_dst_index(vp, &vpi->DstReg),
 					t_dst_mask(vpi->DstReg.WriteMask), t_dst_class(vpi->DstReg.File));
@@ -640,12 +642,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 			o_inst->op=MAKE_VSF_OP(hw_op, t_dst_index(vp, &vpi->DstReg),
 				t_dst_mask(vpi->DstReg.WriteMask), t_dst_class(vpi->DstReg.File));
 			o_inst->src1=t_src(vp, &src[0]);
-			o_inst->src2=MAKE_VSF_SOURCE(t_src_index(vp, &src[0]),
-					SWIZZLE_ONE, SWIZZLE_ONE,
-					SWIZZLE_ONE, SWIZZLE_ONE,
-					t_src_class(src[0].File), VSF_FLAG_NONE);
-
-
+			o_inst->src2=ONE_SRC_0;
 			o_inst->src3=ZERO_SRC_0;
 #endif			
 
@@ -705,7 +702,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[0].Swizzle, 2)),
 					SWIZZLE_ZERO,
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_XYZ : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_XYZ : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			
 			o_inst->src2=MAKE_VSF_SOURCE(t_src_index(vp, &src[1]),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 0)),
@@ -713,7 +710,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[1].Swizzle, 2)),
 					SWIZZLE_ZERO,
 					t_src_class(src[1].File),
-					src[1].NegateBase ? VSF_FLAG_XYZ : VSF_FLAG_NONE);
+					src[1].NegateBase ? VSF_FLAG_XYZ : VSF_FLAG_NONE) | (src[1].RelAddr << 4);
 
 			o_inst->src3=ZERO_SRC_1;
 			goto next;
@@ -726,17 +723,14 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 			o_inst->op=MAKE_VSF_OP(hw_op, t_dst_index(vp, &vpi->DstReg),
 				t_dst_mask(vpi->DstReg.WriteMask), t_dst_class(vpi->DstReg.File));
 			o_inst->src1=t_src(vp, &src[0]);
-			o_inst->src2=MAKE_VSF_SOURCE(t_src_index(vp, &src[0]),
-						SWIZZLE_ONE, SWIZZLE_ONE,
-						SWIZZLE_ONE, SWIZZLE_ONE,
-						t_src_class(src[0].File), VSF_FLAG_NONE);
+			o_inst->src2=ONE_SRC_0;
 			o_inst->src3=MAKE_VSF_SOURCE(t_src_index(vp, &src[1]),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 0)),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 1)),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 2)),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 3)),
 					t_src_class(src[1].File),
-					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE)  | (src[1].RelAddr << 4);
 #else
 			o_inst->op=MAKE_VSF_OP(R300_VPI_OUT_OP_ADD, t_dst_index(vp, &vpi->DstReg),
 					t_dst_mask(vpi->DstReg.WriteMask), t_dst_class(vpi->DstReg.File));
@@ -748,7 +742,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[1].Swizzle, 2)),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 3)),
 					t_src_class(src[1].File),
-					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[1].RelAddr << 4);
 			o_inst->src3=0;
 #endif
 			goto next;
@@ -764,7 +758,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[0].Swizzle, 2)),
 					t_swizzle(GET_SWZ(src[0].Swizzle, 3)),
 					t_src_class(src[0].File),
-					(!src[0].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					(!src[0].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			o_inst->src3=0;
 			goto next;
 			
@@ -791,7 +785,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					VSF_IN_COMPONENT_W,
 					VSF_IN_CLASS_TMP,
 					/* Not 100% sure about this */
-					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE/*VSF_FLAG_ALL*/);
+					(!src[0].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE/*VSF_FLAG_ALL*/);
 
 			o_inst->src3=ZERO_SRC_0;
 			u_temp_i--;
@@ -807,7 +801,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[0].Swizzle, 0)),
 					t_swizzle(GET_SWZ(src[0].Swizzle, 0)),
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			o_inst->src2=ZERO_SRC_0;
 			o_inst->src3=ZERO_SRC_0;
 			goto next;
@@ -822,21 +816,21 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					VSF_IN_COMPONENT_ZERO, // z
 					t_swizzle(GET_SWZ(src[0].Swizzle, 1)), // y
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			o_inst->src2=MAKE_VSF_SOURCE(t_src_index(vp, &src[0]),
 					t_swizzle(GET_SWZ(src[0].Swizzle, 1)), // y
 					t_swizzle(GET_SWZ(src[0].Swizzle, 3)), // w
 					VSF_IN_COMPONENT_ZERO, // z
 					t_swizzle(GET_SWZ(src[0].Swizzle, 0)), // x
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			o_inst->src3=MAKE_VSF_SOURCE(t_src_index(vp, &src[0]),
 					t_swizzle(GET_SWZ(src[0].Swizzle, 1)), // y
 					t_swizzle(GET_SWZ(src[0].Swizzle, 0)), // x
 					VSF_IN_COMPONENT_ZERO, // z
 					t_swizzle(GET_SWZ(src[0].Swizzle, 3)), // w
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			goto next;
 			
 		case OPCODE_DPH://DOT RESULT 1.X Y Z W PARAM 0{} {X Y Z ONE} PARAM 0{} {X Y Z W} 
@@ -849,7 +843,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[0].Swizzle, 2)),
 					VSF_IN_COMPONENT_ONE,
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_XYZ : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_XYZ : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			o_inst->src2=t_src(vp, &src[1]);
 			o_inst->src3=ZERO_SRC_1;
 			goto next;
@@ -869,7 +863,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[0].Swizzle, 0)), // x
 					t_swizzle(GET_SWZ(src[0].Swizzle, 3)), // w
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			
 			o_inst->src2=MAKE_VSF_SOURCE(t_src_index(vp, &src[1]),
 					t_swizzle(GET_SWZ(src[1].Swizzle, 2)), // z
@@ -877,7 +871,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[1].Swizzle, 1)), // y
 					t_swizzle(GET_SWZ(src[1].Swizzle, 3)), // w
 					t_src_class(src[1].File),
-					src[1].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[1].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[1].RelAddr << 4);
 			
 			o_inst->src3=ZERO_SRC_1;
 			o_inst++;
@@ -892,7 +886,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[1].Swizzle, 0)), // x
 					t_swizzle(GET_SWZ(src[1].Swizzle, 3)), // w
 					t_src_class(src[1].File),
-					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					(!src[1].NegateBase) ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[1].RelAddr << 4);
 			
 			o_inst->src2=MAKE_VSF_SOURCE(t_src_index(vp, &src[0]),
 					t_swizzle(GET_SWZ(src[0].Swizzle, 2)), // z
@@ -900,7 +894,7 @@ void r300_translate_vertex_shader(struct r300_vertex_program *vp)
 					t_swizzle(GET_SWZ(src[0].Swizzle, 1)), // y
 					t_swizzle(GET_SWZ(src[0].Swizzle, 3)), // w
 					t_src_class(src[0].File),
-					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE);
+					src[0].NegateBase ? VSF_FLAG_ALL : VSF_FLAG_NONE) | (src[0].RelAddr << 4);
 			
 			o_inst->src3=MAKE_VSF_SOURCE(u_temp_i+1,
 					VSF_IN_COMPONENT_X,
